import { useState, useCallback, useMemo } from 'react';
import { max, scaleTime, scaleLog, extent, line, timeFormat, format } from 'd3';
import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOverlay } from './VoronoiOverlay';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = {
  top: 40,
  right: 20,
  bottom: 80,
  left: 80,
};

const epsilon = 1;

const formatTime = timeFormat("%b %d, %Y");
const formatComma = format(',');

const Tooltip = ({ activeRow, className }) => (
  <text className={className} text-anchor='end' x="-10" y="-10">
    {activeRow.country}: {formatComma(activeRow.deathTotal)} {activeRow.deathTotal > 1 ?'deaths' :'death'} as of {formatTime(activeRow.date)}
  </text>
);

export const LineChart = ({ data, width, height }) => {
  const [
    activeRow,
    setActiveRow,
  ] = useState();

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const allData = useMemo(
    () =>
      data.reduce(
        (accumulator, currentValue) =>
          accumulator.concat(currentValue),
        []
      ),
    [data]
  );
	
  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(allData, xValue))
        .range([0, innerWidth]),
    [allData, xValue, innerWidth]
  );

  const yScale = useMemo(
    () =>
      scaleLog()
        .domain([epsilon, max(allData, yValue)]) // log scale domain cannot start at 0
        .range([innerHeight, 0]),
    [epsilon, allData, yValue, innerHeight]
  );

  const lineGenerator = useMemo(
    () =>
      line()
        .x((d) => xScale(xValue(d)))
        .y((d) => yScale(epsilon + yValue(d))),
    [xScale, yScale, epsilon, xValue, yValue]
  );

  const mostRecentDate = xScale.domain()[1];
	
  const handleVoronoiHover = useCallback(setActiveRow);
  
  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis
          xScale={xScale}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
        />
        <YAxis
          yScale={yScale}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
        />
        <text className="title" y={-10}>
          Global Coronavirus Deaths Over Time by Country
        </text>
        <VoronoiOverlay
          margin={margin}
          onHover={handleVoronoiHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
        />
        {data.map((countryTimeSeries) => {
          //const rgbColor = `rgb(${Math.random()*255}, ${Math.random()*255}, ${Math.random()*255})`;
          return (
            <path
              className="marker-line"
              //stroke={rgbColor}
              d={lineGenerator(countryTimeSeries)}
            />
          );
        })}
        {activeRow ? (
          <>
            <path
              className="marker-line active"
              d={lineGenerator(
                data.find(
                  (countryTimeSeries) =>
                    countryTimeSeries.countryName === activeRow.country
                )
              )}
            />
            <g transform={`translate(${lineGenerator.x()(activeRow)}, ${lineGenerator.y()(activeRow)})`}>
              <circle r={10} />
              <Tooltip activeRow={activeRow} className="tooltip-stroke"/>
              <Tooltip activeRow={activeRow} className="tooltip"/>
            </g>
          </>
        ) : null}
      </g>
    </svg>
  );
};
